QMPlay_Tag::QMPlay_Tag( const char *_fileName )
{
	using namespace TagLib;

	fileName = strdup( _fileName );

	isNull = true;

	//General tags
	{
		FileRef f( fileName, false );
		if ( !f.isNull() && f.tag() )
		{
			Tag *tag = f.tag();

			title = tag->title();
			artist = tag->artist();
			album = tag->album();
			comment = tag->comment();
			genre = tag->genre();
			year = tag->year();
			track = tag->track();

			isNull = false;
		}
	}

	//ID3v2 picture
	{
		MPEG::File f( fileName, false );
		ID3v2::Tag *id3v2 = f.ID3v2Tag();
		if ( id3v2 && !id3v2->isEmpty() )
		{
			ID3v2::FrameList pict = id3v2->frameList( "APIC" );
			if ( !pict.isEmpty() )
			{
				picture = ID3v2::AttachedPictureFrame( pict.front()->render() ).picture();
				isNull = false;
			}
		}
	}
}
QMPlay_Tag::~QMPlay_Tag()
{
	free( fileName );
}

void QMPlay_Tag::clear()
{
	title = "";
	artist = "";
	album = "";
	comment = "";
	genre = "";
	year = track = 0;
	picture.clear();

	save();

	using namespace TagLib;

	//all ID3 tags
	{
		MPEG::File f( fileName, false );
		f.save( MPEG::File::NoTags );
	}
}

void QMPlay_Tag::save()
{
	using namespace TagLib;

	//General tag
	{
		FileRef f( fileName, false );
		if ( !f.isNull() && f.tag() )
		{
			Tag *tag = f.tag();

			tag->setTitle( title );
			tag->setArtist( artist );
			tag->setAlbum( album );
			tag->setComment( comment );
			tag->setGenre( genre );
			tag->setYear( year );
			tag->setTrack( track );

			f.save();
		}
	}

	//ID3v2 picture
	{
		MPEG::File f( fileName, false );
		ID3v2::Tag *id3v2 = f.ID3v2Tag();
		if ( id3v2 && !id3v2->isEmpty() )
		{
			ID3v2::FrameList pict = id3v2->frameList( "APIC" );
			while ( pict.size() )
				id3v2->removeFrame( pict[ 0 ] );
			if ( !picture.isEmpty() )
			{
				ID3v2::AttachedPictureFrame *pictFrame = new ID3v2::AttachedPictureFrame;
				pictFrame->setPicture( picture );
				QString mimeType = "image/" + QImageReader( QDataStream( QByteArray( picture.data(), picture.size() ) ).device() ).format();
				pictFrame->setMimeType( mimeType.toAscii().data() );
				id3v2->addFrame( pictFrame );
			}
			f.save();
		}
	}
}
